/*------------------------------------------------------------------------------------
   Name: Matrix-Signals.mq4
   Author: Xaphod. 2012, xaphod.com
   
   Description: Signal/alert generating indicator for use with the 'Genesis Matrix
                Trading System'
   Requires:  experts/indicators/TVI.mq4
              experts/indicators/GannHiLo-Histo.mq4
              experts/indicators/T3MA-Basic.mq4
              experts/libraries/GMTS-Interface.dll
              experts/files/GMTSSignals.exe
              experts/files/GMTSalert.wav  
   Change log:
       2012-11-11. Xaphod, v1.01
          - Added Symbol Prefix parameter.
          - Remove symbol prefix, suffix and non alphanumeric chars from symbol string 
            sent to alert program.
          - Adjust the size of the alert window to account for the desktop font size 
            variations.
          - Read out 5 digit symbols.
          - Check if all files are installed on start up and check for dlls enabled.
          - Improved symbol validation function. 
          - Added blink time parameter in '../experts/files/MatrixSignals.ini' to 
            adjust the alert blink time. default BlinkTime=60. One will have to edit
            the ini file to change the value.

       2012-08-31. Xaphod, v1.00
          - Initial release
-------------------------------------------------------------------------------------*/
// Indicator properties
#property copyright "Copyright 2012, Xaphod"
#property link      "http://www.xaphod.com"

#property indicator_chart_window

//#include <xPrint.mqh>

// Win32 API
#import "Kernel32.dll"
  int CreateFileA(string lpFileName,int dwDesiredAccess, int dwShareMode, int lpSecurityAttributes, int dwCreationDisposition,int dwFlagsAndAttributes, int hTemplateFile);
  int CloseHandle(int hObject);
#import

//#import "c:\bin\projects\MatrixSignals\Bridge\MatrixBridge.dll"
#import "MatrixBridge.dll"
  int gmaInit(int hChartWnd);
  int gmaDeinit(int hChartWnd);
  int gmaSymbolAdd(string sSymbol);
  int gmaSymbolUpdate(int iSymbolId, int iSignal);
#import

#define OPEN_EXISTING 3
#define FILE_SHARE_READ 1
#define GENERIC_READ 0x80000000
#define FILE_ATTRIBUTE_NORMAL 128
#define INVALID_HANDLE_VALUE 0xFFFFFFFF

// Constant definitions
#define INDICATOR_NAME "Matrix-Signals"
#define INDICATOR_VERSION "1.01"

#define IND_TVI           "TVI"
#define IND_GANNHILO      "GannHiLo-Histo"
#define IND_T3MA          "T3MA-Basic"
#define FILE_ALERTBRIDGE  "MatrixBridge.dll"
#define FILE_ALERTPROGRAM "MatrixSignals.exe"
#define FILE_ALERTSND     "MatrixAlert.wav"

// Signals
#define SIG_UP 1
#define SIG_NO 0
#define SIG_DN -1

#define MAX_SYMBOLS 20

// Indicator parameters
extern string    Indi.Version=INDICATOR_VERSION;
extern string    TimeFrame.Settings="";
extern int       TimeFrame.Period=0;                         // Timeframe: 0,1,5,15,30,60,240,1440 etc. Current Timeframe=0. 
extern string    Symbol.Settings="";
extern string    Symbol.List="EURUSD,EURJPY,AUDUSD,GBPUSD";  // List of symbols to monitor. , without the brokers prefix and suffix.
extern string    Symbol.Suffix="";                           // Broker symbol suffix. Ex m for EURUSDm.
extern string    Symbol.Prefix="";                           // Broker symbol prefix. Ex fx_ for fx_EURUSD.
extern string    TVI.Settings="";
extern int       TVI.r=12;
extern int       TVI.s=12;
extern int       TVI.u=5;
extern string    CCI.Settings="";
extern int       CCI.Period=20;
extern string    GannHiLo.Settings="";
extern int       GannHiLo.Period=10;
extern string    T3MA.Settings="";
extern int       T3MA.Period=8;
extern double    T3MA.b=0.618;

// Global module varables
// Histogram
  string saSymbol[MAX_SYMBOLS];
  int iaSymbol[MAX_SYMBOLS];
  int iaSignal[MAX_SYMBOLS];
  int iaBars[MAX_SYMBOLS];
  int iSymbolCount=0;

// Globals
int ghChartWnd;

//-----------------------------------------------------------------------------
// function: init()
// Description: Custom indicator initialization function.
//-----------------------------------------------------------------------------
int init() {  

  if (!IsDllsAllowed())
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+", \'Allow Dll imports\' is required for this indicator to work. ");

  // Check if external indicators and files are installed
  if (!IndicatorExists(IND_TVI+".ex4"))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalPath()+"\\experts\\indicators\\"+IND_TVI+".ex4 cannot be found.");
  if (!IndicatorExists(IND_GANNHILO+".ex4"))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalPath()+"\\experts\\indicators\\"+IND_GANNHILO+".ex4 cannot be found.");
  if (!IndicatorExists(IND_T3MA+".ex4"))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalPath()+"\\experts\\indicators\\"+IND_T3MA+".ex4 cannot be found.");
  if (!FileExists(TerminalPath()+"\\experts\\libraries\\"+FILE_ALERTBRIDGE))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalPath()+"\\experts\\libraries\\"+FILE_ALERTBRIDGE+" cannot be found.");
  if (!FileExists(TerminalPath()+"\\experts\\files\\"+FILE_ALERTPROGRAM))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalPath()+"\\experts\\files\\"+FILE_ALERTPROGRAM+" cannot be found.");
  if (!FileExists(TerminalPath()+"\\experts\\files\\"+FILE_ALERTSND))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalPath()+"\\experts\\files\\"+FILE_ALERTSND+" cannot be found.");
 
  // Sanitize string inputs
  Symbol.List=StringTrimRight(StringTrimLeft(Symbol.List));
  Symbol.Suffix=StringTrimRight(StringTrimLeft(Symbol.Suffix));
  Symbol.Prefix=StringTrimRight(StringTrimLeft(Symbol.Prefix)); 
  
  // Misc
  IndicatorShortName(INDICATOR_NAME+"("+TF2Str(TimeFrame.Period)+")");
  ghChartWnd=0;
 
  return(0);
}


//-----------------------------------------------------------------------------
// function: deinit()
// Description: Custom indicator deinitialization function.
//-----------------------------------------------------------------------------
int deinit() {
  // Chart change or parameter change. Do not clean up.
  gmaDeinit(ghChartWnd);
  iSymbolCount=0;
  return (0);
}


//-----------------------------------------------------------------------------
// function: start()
// Description: Custom indicator iteration function.
//-----------------------------------------------------------------------------
int start() {
  static string saSymbol[MAX_SYMBOLS];
  static int iaSymbol[MAX_SYMBOLS];
  static int iaSignal[MAX_SYMBOLS];
  static int iaBars[MAX_SYMBOLS];
  static int iSymbolCount=0;
  int i,iNewBars;
  
  /*
  string s="_UK100";
  PrintD(s+" = "+StrToAlphaNum(s));
  s="#USIDX";
  PrintD(s+" = "+StrToAlphaNum(s));
  s="DE100#";
  PrintD(s+" = "+StrToAlphaNum(s));
  s="fx_EURUSD";
  PrintD(s+" = "+SanitizeSymbol(s));
  s="EURUSD.arm";
  PrintD(s+" = "+SanitizeSymbol(s));
  s="EURUSDm";
  PrintD(s+" = "+SanitizeSymbol(s));
  s="_UK100";
  PrintD(s+" = "+SanitizeSymbol(s));
  return(0);
  */
  
  // Init symbols and load GUI;
  if (iSymbolCount==0) {
    // Check if chart is running
    if (IsStopped())
      return(0);
    ghChartWnd=WindowHandle(Symbol(), Period());  
    if (ghChartWnd==0)
      return(0);
     
    // Load symbols
    iSymbolCount=LoadSymbols(Symbol.List, Symbol.Prefix, Symbol.Suffix, saSymbol, iaSymbol); 
    // Error. No symbols found
    if (iSymbolCount==0) {
      Alert(INDICATOR_NAME+", "+Symbol()+". Error! No Symbols Found");
      iSymbolCount--;
      return(0);
    }
    // Start GUI and add symbols to it  
    else {
      if (gmaInit(ghChartWnd)>0) {
        for (i=0; i<iSymbolCount; i++)
          iaSymbol[i]=gmaSymbolAdd(SanitizeSymbol(saSymbol[i]));
      }
      else {
        Alert(INDICATOR_NAME+", "+Symbol()+". Error! Unable to start GUI");
        iSymbolCount--;
        return(0);
      }
    }
  }
  
  // Get Signals
  if (iSymbolCount>0) {
    for (i=0;i<iSymbolCount;i++) {
      iNewBars=NewBars(TimeFrame.Period, i, iaBars, saSymbol);
      if (iNewBars>0) {
        iaSignal[i]=GetCurrentSignal(saSymbol[i],TimeFrame.Period);
        gmaSymbolUpdate(iaSymbol[i],iaSignal[i]);
      }      
    }
  }
  
  return(0);
}
//+------------------------------------------------------------------+


//-----------------------------------------------------------------------------
// function: LoadSymbols()
// Description: Load the symbols from list provided
//-----------------------------------------------------------------------------
int LoadSymbols(string sSymbolList, string sSymbolPrefix, string sSymbolSuffix, string& saSymbol[], int& iaSymbol[] ) {
  int iSymbolCount=0,i=0;
  string sCurSymbol;
  
  //Get list of symbols to use
  while (StringLen(sSymbolList)>0 && i<MAX_SYMBOLS) {
    sCurSymbol=ParseStr(sSymbolList);
    // Add symbol from parameters to symbol list
    if (StringLen(sCurSymbol)>0) {
      if (IsValidSymbol(sSymbolPrefix+sCurSymbol+sSymbolSuffix)) {
        saSymbol[i]=sSymbolPrefix+sCurSymbol+sSymbolSuffix;
        i++;
        iSymbolCount++;
      }
      else {
        Alert(INDICATOR_NAME+", "+Symbol()+". Error! "+sCurSymbol+sSymbolSuffix+" is not a valid symbol.");
      }  
    }  
  }
  return(iSymbolCount);  
}


//-----------------------------------------------------------------------------
// function: GetCurrentSignals()
// Description: Load the current signals into the signal array
//-----------------------------------------------------------------------------
int GetCurrentSignal(string sSymbol, int iTimeFrame){
  int i,iDataError;
  int iCurTVI, iCurCCI, iCurGHL, iCurT3;
  double dUp, dDn, dSig1, dSig2;
  
  // TVI
  dSig1=iCustom(sSymbol,iTimeFrame,IND_TVI,TVI.r,TVI.s,TVI.u,0,1);
  dSig2=iCustom(sSymbol,iTimeFrame,IND_TVI,TVI.r,TVI.s,TVI.u,0,2);
      
  if (iClose(sSymbol,iTimeFrame,1)==0 || iClose(sSymbol,iTimeFrame,1)==EMPTY_VALUE)
    iCurTVI=SIG_NO;
  else  if (dSig1>=dSig2)   
    iCurTVI=SIG_UP;
  else 
    iCurTVI=SIG_DN;
  
  // T3 MA
  dSig1 = iCustom(sSymbol,iTimeFrame,IND_T3MA,T3MA.Period, T3MA.b,300,0,1);
  dSig2 = iCustom(sSymbol,iTimeFrame,IND_T3MA,T3MA.Period, T3MA.b,300,0,2);
  if (iClose(sSymbol,iTimeFrame,1)==0 || iClose(sSymbol,iTimeFrame,1)==EMPTY_VALUE)
    iCurT3=SIG_NO;
  else if (dSig1>=dSig2) 
    iCurT3=SIG_UP;
  else
    iCurT3=SIG_DN;
  
  // Gann HiLo
  dUp = iCustom(sSymbol,iTimeFrame,IND_GANNHILO,GannHiLo.Period,0,1);
  if (iClose(sSymbol,iTimeFrame,1)==0 || iClose(sSymbol,iTimeFrame,1)==EMPTY_VALUE)
    iCurGHL=SIG_NO;
  else if (dUp>0) 
    iCurGHL=SIG_UP;
  else
    iCurGHL=SIG_DN;
  
  //Trend
  dSig1=iCCI(sSymbol, iTimeFrame, CCI.Period, PRICE_TYPICAL,1);
  if (iClose(sSymbol,iTimeFrame,1)==0 || iClose(sSymbol,iTimeFrame,1)==EMPTY_VALUE)
    iCurCCI=SIG_NO;
  else if (dSig1<0) 
    iCurCCI=SIG_DN;
  else
    iCurCCI=SIG_UP;
  
  // Set signal
  i=UpSignal(iCurTVI, iCurCCI, iCurGHL, iCurT3);
  if (i>2)
    return(i);
  i=DnSignal(iCurTVI, iCurCCI, iCurGHL, iCurT3);
  if (i<-2)
    return(i);  

  return(0);
}


//-----------------------------------------------------------------------------
// function: UpSignal()
// Description: 
//-----------------------------------------------------------------------------
int UpSignal(int iCurTVI, int iCurCCI, int iCurGHL, int iCurT3) {
  int n=0;
  if (iCurTVI==SIG_UP) n++;
  if (iCurCCI==SIG_UP) n++;
  if (iCurGHL==SIG_UP) n++;
  if (iCurT3==SIG_UP) n++;  
  return(n);
}


//-----------------------------------------------------------------------------
// function: DnSignal()
// Description: 
//-----------------------------------------------------------------------------
int DnSignal(int iCurTVI, int iCurCCI, int iCurGHL, int iCurT3) {
  int n=0;
  if (iCurTVI==SIG_DN) n--;
  if (iCurCCI==SIG_DN) n--;
  if (iCurGHL==SIG_DN) n--;
  if (iCurT3==SIG_DN) n--;  
  return(n);
}


//-----------------------------------------------------------------------------
// function: ParseStr()
// Description: Get next value in a comma delimited string
//-----------------------------------------------------------------------------
string ParseStr(string& sData, string sDelimiter=",") {
  int iPos;
  string sResult;
  
  iPos=StringFind(sData,sDelimiter,0);
  if (iPos==-1 && sData!="") {
    sResult=sData;
    sData="";
    return(sResult);
  }
  sResult=StringSubstr(sData,0,iPos);
  sData=StringSubstr(sData,iPos+1);
  return(sResult);
}


//-----------------------------------------------------------------------------
// function: NewBars()
// Description: Return nr of new bars on a TF
//-----------------------------------------------------------------------------
int NewBars(int iPeriod, int n, int& iaBars[], string& saSymbol[]) {
  int iNewSize;
  int iNewBars;
  datetime tTimeArray[];
  
  ArrayCopySeries(tTimeArray,MODE_TIME,saSymbol[n],iPeriod);
  iNewSize=ArraySize(tTimeArray);
  iNewBars=iNewSize-iaBars[n];
  iaBars[n]=iNewSize;
  return(iNewBars);
}


//-----------------------------------------------------------------------------
// function: IndicatorExists()
// Description: Check if an indicator exists
//-----------------------------------------------------------------------------
bool IndicatorExists(string sIndicatorName) {
  int hFile;
  string sFile;
    
  // Exit if dlls are disabled
  if (!IsDllsAllowed())
    return(True);
  
  // Try to open indicator 
  sFile=TerminalPath()+"\\experts\\indicators\\"+sIndicatorName;
  hFile=CreateFileA(sFile,0,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);  
  if (hFile==INVALID_HANDLE_VALUE) {
    return(False);    
  }
  else { 
    CloseHandle(hFile);
    return(True);
  }
  return(False);
}


//-----------------------------------------------------------------------------
// function: FileExists()
// Description: Check if a file exists
//-----------------------------------------------------------------------------
bool FileExists(string sFile) {
  int hFile;
    
  // Exit if dlls are disabled
  if (!IsDllsAllowed())
    return(True);
  
  // Try to open file 
  hFile=CreateFileA(sFile,0,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);  
  if (hFile==INVALID_HANDLE_VALUE) {
    return(False);    
  }
  else { 
    CloseHandle(hFile);
    return(True);
  }
  return(False);
}


//-----------------------------------------------------------------------------
// function: IsValidSymbol()
// Description: Get symbol that contains the given symbol string by scanning
//              the file 'symbols.raw'. 
//              Returns true if symbol is found.
//-----------------------------------------------------------------------------
bool IsValidSymbol(string sSymbol) {
   int hSymbolFile;
   int iNrOfSymbols;
   string sResult;
   int i;
   
   hSymbolFile=FileOpenHistory("symbols.raw",FILE_BIN|FILE_READ);
   iNrOfSymbols=FileSize(hSymbolFile)/1936;   
   for (i=0;i<iNrOfSymbols;i++) {
     if(FileIsEnding(hSymbolFile)) 
       break;
     sResult=StringTrimRight(FileReadString(hSymbolFile,12));
     if (sResult==sSymbol) {        
       FileClose(hSymbolFile);
       return(True); 
     }  
     sResult="";       
     FileSeek(hSymbolFile,1936-12,SEEK_CUR);     
   }
   FileClose(hSymbolFile);
   return(False);   
}


//-----------------------------------------------------------------------------
// function: TF2Str()
// Description: Convert time-frame to a string
//-----------------------------------------------------------------------------
string TF2Str(int iPeriod) {
  switch(iPeriod) {
    case PERIOD_M1: return("M1");
    case PERIOD_M5: return("M5");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1");
    case PERIOD_H4: return("H4");
    case PERIOD_D1: return("D1");
    case PERIOD_W1: return("W1");
    case PERIOD_MN1: return("MN1");
    default: return("M"+iPeriod);
  }
  return(0);
}


//-----------------------------------------------------------------------------
// function: StrToAlphaNum()
// Description: Convert a string to an alphanumeric string
//-----------------------------------------------------------------------------
string SanitizeSymbol(string sSymbol) {
  string sResult="";
  int i;
  
  sResult=sSymbol;
  if (StringLen(Symbol.Suffix)>0)
    sResult=StringSubstr(sResult,0,StringLen(sResult)-StringLen(Symbol.Suffix));
  if (StringLen(Symbol.Prefix)>0)
   sResult=StringSubstr(sResult,StringLen(Symbol.Prefix),StringLen(sResult)-StringLen(Symbol.Prefix));
  sResult=StrToAlphaNum(sResult);
  
  return(sResult);
}


//-----------------------------------------------------------------------------
// function: StrToAlphaNum()
// Description: Convert a string to an alphanumeric string
//-----------------------------------------------------------------------------
string StrToAlphaNum(string sInStr) {
  string sResult="";
  int i;
  
  for (i=0; i<StringLen(sInStr);i++)
    if (IsAlphaNum(StringGetChar(sInStr,i)))
      sResult=sResult+StringSubstr(sInStr,i,1);
  
  return(sResult);
}


//-----------------------------------------------------------------------------
// function: IsDigit()
// Description: Is the character a digit?
//-----------------------------------------------------------------------------
bool IsDigit(int ch) {
  if (ch>='0' && ch<='9')
    return(true);
  else  
    return(False);
}


//-----------------------------------------------------------------------------
// function: IsAlpha()
// Description: Is the character an english letter? (A-Z and a-z)
//-----------------------------------------------------------------------------
bool IsAlpha(int ch) {
  if (ch>='A' && ch<='Z')
    return(true);
  else if (ch>='a' && ch<='z')
    return(true);
  else  
    return(False);
}    


//-----------------------------------------------------------------------------
// function: IsAlphaNum()
// Description: Is the character an alphanumeric character? (0-9, A-Z, a-z)
//-----------------------------------------------------------------------------
bool IsAlphaNum(int ch) {
  
  if (IsAlpha(ch) || IsDigit(ch))
    return(true);
  else 
    return(False);
} 